from django.db import models
from reqman.apps.account.models import User

from pygments.formatters.html import HtmlFormatter
from pygments import highlight


MAIN_CHOICES = ( ("",""),   ("M1", "M1 : system/function shall [not] set [<quantifier>] item [to stateValue]"), 
                            ("M2", "M2 : system/function shall [not] set state to stateValue"), 
                            ("M3", "M3 : system/function shall [not] send [<quantifier>] item"),
                            ("M4", "M4 : system/function shall [not] receive [<quantifier>] item"),
                            ("M5", "M5 : system/function shall [not] ingest flow"),
                            ("M6", "M6 : system/function shall [not] emit flow"),
                            ("M7", "M7 : system shall [not] perform function"),
                            ("M8", "M8 : function shall [not] invoke function"),
                            ("M9", "M9 : system shall [not] present interface [to system]"),
                            ("M10", "M10 : system/interface shall [not] transfer flow/item"),
                            ("M11", "M11 : system shall [not] interact with system [using <connection>] [as dened in <text>]"),
                            ("M12", "M12 : system shall [not] have state state [with values stateSet]"),
                            ("M13", "M13 : system state stateValue shall [not] have substate state [with values stateSet]"),
                            ("M14", "M14 : system item shall [not] take values from stateSet"),
                            ("M15", "M15 : system item shall [not] be composed from item"),
                            ("M16", "M16 : system shall [not] contain system")
                        )

#to be updated with domains from DSO
SYSTEM_OR_FUNCTION_CHOICES = (("SYSTEM", "SYSTEM"), ("FUNCTION", "FUNCTION"))
SHALL_CHOICES = ( ("shall", "SHALL"), ("shall not", "SHALL NOT"))
QUANTIFIER_CHOICES = ( ("none","NONE"), ("all", "ALL"), ("only", "ONLY"),
                                ("more than", "MORE THAN"), ("less than", "LESS THAN"), ("exactly","EXACTLY"), 
                                ("at least", "AT LEAST"), ("at most", "AT MOST")
                     )
NUMBER_UNITS_CHOICES = (("",""), ("meters","METERS"), ("kilometers","KILOMETERS"), ("volt","VOLT"))
ITEM_CHOICES = (("ITEM", "ITEM"), ("ITEM2", "ITEM2") )
STATE_VALUE_CHOICES = (("enabled", "ENABLED"), ("disabled", "DISABLED"))


def create_main(instance):
    '''Create the Main Model for the Boilerplate'''
    try :
        main = Main.objects.get(boilerplate_of_main = instance, main_owner = instance.owner)
        if instance.has_main:
            return
        main = Main.objects.get(boilerplate_of_main = instance, main_owner = instance.owner)
        main.delete()
    except Main.DoesNotExist:
        if instance.has_main:
            main = Main(boilerplate_of_main = instance, main_owner = instance.owner)
            main.save()


def create_m1(instance, now_main, prev_main):
    try :
        main = Main_M1.objects.get(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M1.objects.get(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[1][0]:
            main = Main_M1(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
            main.save()
    except Main_M1.DoesNotExist:
        if now_main == MAIN_CHOICES[1][0]:
            main = Main_M1(main_owner_of_m1 = instance, boilerplate_of_m1 = instance.boilerplate_of_main)
            main.save()


def create_m2(instance, now_main, prev_main):
    try :
        main = Main_M2.objects.get(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M2.objects.get(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[2][0]:
            main = Main_M2(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
            main.save()
    except Main_M2.DoesNotExist:
        if now_main == MAIN_CHOICES[2][0]:
            main = Main_M2(main_owner_of_m2 = instance, boilerplate_of_m2 = instance.boilerplate_of_main)
            main.save()


def create_m3(instance, now_main, prev_main):
    try :
        main = Main_M3.objects.get(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M3.objects.get(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[3][0]:
            main = Main_M3(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
            main.save()
    except Main_M3.DoesNotExist:
        if now_main == MAIN_CHOICES[3][0]:
            main = Main_M3(main_owner_of_m3 = instance, boilerplate_of_m3 = instance.boilerplate_of_main)
            main.save()


def create_m4(instance, now_main, prev_main):
    try :
        main = Main_M4.objects.get(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
        if now_main == prev_main:
            return
        main = Main_M4.objects.get(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
        main.delete()
        if now_main == MAIN_CHOICES[4][0]:
            main = Main_M4(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
            main.save()
    except Main_M4.DoesNotExist:
        if now_main == MAIN_CHOICES[4][0]:
            main = Main_M4(main_owner_of_m4 = instance, boilerplate_of_m4 = instance.boilerplate_of_main)
            main.save()



class Main(models.Model):
    boilerplate_of_main = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate', on_delete=models.CASCADE)
    main_owner = models.ForeignKey(User, related_name='main_owner', on_delete=models.CASCADE)
    main_choices = models.CharField(choices=MAIN_CHOICES, max_length=100)

    

    def __init__(self, *args, **kwargs):
        super(Main, self).__init__(*args, **kwargs)
        self.__important_fields = ['main_choices']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))
        

    def save(self, **kwargs):
        """Create P1, P2 or P3 """
        prev_main_list = []
        super(Main, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_main = getattr(self, orig)
            prev_main_list.append(previous_main)        
        '''Create [M1 - M16]'''
        create_m1( self, self.main_choices, prev_main_list[0])
        create_m2( self, self.main_choices, prev_main_list[0])
        create_m3( self, self.main_choices, prev_main_list[0])
        create_m4( self, self.main_choices, prev_main_list[0])
           

    class Meta:
        ordering = ['boilerplate_of_main']


class Main_M1(models.Model):
    boilerplate_of_m1 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M1', on_delete=models.CASCADE)
    main_owner_of_m1 = models.ForeignKey(Main, related_name='main_owner_of_M1', on_delete=models.CASCADE)
    sys_fun_m1 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m1 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    quantifier_m1 = models.CharField(choices=QUANTIFIER_CHOICES, max_length=100)
    numerical_m1 = models.IntegerField(default=0, blank=True) 
    mumerical_units_m1 = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    item_m1 = models.CharField(choices=ITEM_CHOICES, max_length=100)
    statevalue_m1 = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100)
        

    class Meta:
        ordering = ['boilerplate_of_m1']


class Main_M2(models.Model):
    boilerplate_of_m2 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M2', on_delete=models.CASCADE)
    main_owner_of_m2 = models.ForeignKey(Main, related_name='main_owner_of_M2', on_delete=models.CASCADE)
    sys_fun_m2 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m2 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    state_m2 = models.CharField(default = ' ', max_length=100)
    statevalue_m2 = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100)
        

    class Meta:
        ordering = ['boilerplate_of_m2']


class Main_M3(models.Model):
    boilerplate_of_m3 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M3', on_delete=models.CASCADE)
    main_owner_of_m3 = models.ForeignKey(Main, related_name='main_owner_of_M3', on_delete=models.CASCADE)
    sys_fun_m3 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m3 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    quantifier_m3 = models.CharField(choices=QUANTIFIER_CHOICES, max_length=100)
    numerical_m3 = models.IntegerField(default=0, blank=True) 
    mumerical_units_m3 = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    item_m3 = models.CharField(choices=ITEM_CHOICES, max_length=100)
   

    class Meta:
        ordering = ['boilerplate_of_m3']


class Main_M4(models.Model):
    boilerplate_of_m4 = models.ForeignKey('reqtool.Boilerplate', related_name='main_boilerplate_of_M4', on_delete=models.CASCADE)
    main_owner_of_m4 = models.ForeignKey(Main, related_name='main_owner_of_M4', on_delete=models.CASCADE)
    sys_fun_m4 = models.CharField(choices=SYSTEM_OR_FUNCTION_CHOICES, max_length=100)
    shall_m4 = models.CharField(choices=SHALL_CHOICES, max_length=100)
    quantifier_m4 = models.CharField(choices=QUANTIFIER_CHOICES, max_length=100)
    numerical_m4 = models.IntegerField(default=0, blank=True) 
    mumerical_units_m4 = models.CharField(choices=NUMBER_UNITS_CHOICES, max_length=100)
    item_m4 = models.CharField(choices=ITEM_CHOICES, max_length=100)
   

    class Meta:
        ordering = ['boilerplate_of_m4']




