from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.main_req import Main, Main_M1, Main_M2, Main_M3, Main_M4
from reqman.apps.reqtool.rest_api.serializers.main_req import MainSerializer, Main_M1Serializer, Main_M2Serializer, Main_M3Serializer, Main_M4Serializer
from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki

class MainListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = MainSerializer
    #
    queryset = Main.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main = Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class MainDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = MainSerializer
    
    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main= Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Main_M1ListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M1Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M1.objects.filter(boilerplate_of_m1 = bp, main_owner_of_m1 = mn)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Main_M1DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M1Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M1.objects.filter(boilerplate_of_m1 = bp, main_owner_of_m1 = mn)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Main_M2ListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M2Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M2.objects.filter(boilerplate_of_m2 = bp, main_owner_of_m2 = mn)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Main_M2DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M2Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M2.objects.filter(boilerplate_of_m2 = bp, main_owner_of_m2 = mn)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Main_M3ListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M3Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M3.objects.filter(boilerplate_of_m3 = bp, main_owner_of_m3 = mn)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Main_M3DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M3Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M3.objects.filter(boilerplate_of_m3 = bp, main_owner_of_m3 = mn)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class Main_M4ListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M4Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M4.objects.filter(boilerplate_of_m4 = bp, main_owner_of_m4 = mn)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class Main_M4DetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = Main_M4Serializer

    def get_queryset(self):
        bp = self.kwargs['boilerplate']
        mn = self.kwargs['mains']
        #print(group_pk)
        queryset_main = Main_M4.objects.filter(boilerplate_of_m4 = bp, main_owner_of_m4 = mn)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()





